
/*
 * Name: Bhaskar S
 * 
 * Date: 10/10/2009 
 */

package com.polarsparc.jdk.compiler;

import java.util.*;

public class CompiledFilter {
	private static int[] nums;
	private static Filter func;
	
	public static void main(String[] args) {
		if (args.length != 1) {
			System.out.printf("Usage: java %s <DynFilterEvenGt500 | DynFilterOddLt500>\n", CompiledFilter.class.getName());
			System.exit(1);
		}
		init(args[0]);
		execute();
	}
	
	public static void init(String name) {
		try {
			nums = new int[100];
			Random rand = new Random();
			for (int i = 0; i < 100; ++i) {
				nums[i] = rand.nextInt(1000);
			}
			
			DynamicCompiler compiler = new DynamicCompiler();
			compiler.init();
			
			Class<?> clazz = null;
			if (name.equals("DynFilterEvenGt500")) {
				clazz = compiler.compileToClass("com.polarsparc.jdk.compiler." + name, DynFilterEvenGt500());
			}
			else {
				clazz = compiler.compileToClass("com.polarsparc.jdk.compiler." + name, DynFilterOddLt500());
			}
			
			func = (Filter) clazz.newInstance();
		}
		catch (Throwable ex) {
			ex.printStackTrace(System.err);
			System.exit(1);
		}
	}
	
	public static void execute() {
		try {
			long stm = System.currentTimeMillis();
			for (int n : nums) {
				if (func.filter(n)) {
					System.out.printf("CompiledFilter: %d\n", n);
				}
			}
			long etm = System.currentTimeMillis();
			System.out.printf("CompiledFilter execute time: %d ms\n", (etm-stm));
		}
		catch (Throwable ex) {
			ex.printStackTrace(System.err);
			System.exit(1);
		}
	}
	
	private static String DynFilterEvenGt500() {
		StringBuilder sb = new StringBuilder();
		sb.append("package com.polarsparc.jdk.compiler;\n");
		sb.append("public class DynFilterEvenGt500 implements Filter {\n");
		sb.append("   @Override\n");
		sb.append("   public boolean filter(int num) {\n");
		sb.append("      if (num > 500 && (num % 2) == 0) {\n");
		sb.append("         return true;\n");
		sb.append("      }\n");
		sb.append("      return false;\n");
		sb.append("   }\n");
		sb.append("}\n");
		return sb.toString();
	}
	
	private static String DynFilterOddLt500() {
		StringBuilder sb = new StringBuilder();
		sb.append("package com.polarsparc.jdk.compiler;\n");
		sb.append("public class DynFilterOddLt500 implements Filter {\n");
		sb.append("   @Override\n");
		sb.append("   public boolean filter(int num) {\n");
		sb.append("      if (num < 500 && (num % 2) != 0) {\n");
		sb.append("         return true;\n");
		sb.append("      }\n");
		sb.append("      return false;\n");
		sb.append("   }\n");
		sb.append("}\n");
		return sb.toString();
	}
}
