
/*
 * Name: Bhaskar S
 * 
 * Date: 10/10/2009 
 */

package com.polarsparc.jdk.compiler;

import java.util.*;
import javax.script.*;

public class ScriptVsCompiled {
	private static int[] nums;
	
	public static void main(String[] args) {
		init();
		ScriptClient sc = new ScriptClient();
		sc.init();
		sc.execute();
		CompiledClient cc = new CompiledClient();
		cc.init();
		cc.execute();
	}
	
	public static void init() {
		try {
			nums = new int[100];
			Random rand = new Random();
			for (int i = 0; i < 100; ++i) {
				nums[i] = rand.nextInt(1000);
			}
		}
		catch (Throwable ex) {
			ex.printStackTrace(System.err);
			System.exit(1);
		}
	}
	
	private static class ScriptClient {
		private ScriptEngine seng;
		
		void init() {
			ScriptEngineManager smgr = new ScriptEngineManager();
			seng = smgr.getEngineByExtension("js");
			if (seng == null) {
				System.err.println("*** Could not find engine - Rhino !!!");
				System.exit(1);
			}
		}
		
		void execute() {
			try {
				Bindings bnds = seng.getBindings(ScriptContext.ENGINE_SCOPE);
				long stm = System.currentTimeMillis();
				for (int n : nums) {
					bnds.put("num", n);
					if ((Boolean)seng.eval("if (num > 500 && (num % 2) == 0) { true; } else { false; }")) {
						System.out.printf("ScriptClient: %d\n", n);
					}
				}
				long etm = System.currentTimeMillis();
				System.out.printf("ScriptCliient execute time: %d ms\n", (etm-stm));
			}
			catch (Throwable ex) {
				ex.printStackTrace(System.err);
				System.exit(1);
			}
		}
	}
	
	private static class CompiledClient {
		private Filter func;
		
		void init() {
			try {
				Class<?> clazz = Class.forName("com.polarsparc.jdk.compiler.FilterEvenGt500");
				func = (Filter) clazz.newInstance();
			}
			catch (Throwable ex) {
				ex.printStackTrace(System.err);
				System.exit(1);
			}
		}
		
		void execute() {
			try {
				long stm = System.currentTimeMillis();
				for (int n : nums) {
					if (func.filter(n)) {
						System.out.printf("CompiledClient: %d\n", n);
					}
				}
				long etm = System.currentTimeMillis();
				System.out.printf("CompiledClient execute time: %d ms\n", (etm-stm));
			}
			catch (Throwable ex) {
				ex.printStackTrace(System.err);
				System.exit(1);
			}
		}
	}
}
